#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>

int main(int argc, char *argv[])
{
	int fd;
	char buf[1024];
	ssize_t len;
	int ret;

	if (argc < 2) {
		printf("usage: %s <file name>\n", argv[0]);
		return EXIT_SUCCESS;
	}

	/*
	 * open() は新しいファイル・ディスクリプタを返す。エラーが発生
	 * した場合は -1 が返され、errno が適切に設定される。
	 */
	fd = open(argv[1], O_RDONLY);
	/* エラーが発生した? */
	if (fd == -1) {
		/* errno に応じたエラーメッセージを出力する */
		perror("open");
		return EXIT_FAILURE;
	}

	ret = EXIT_SUCCESS;
	for(;;) {
		/*
		 * 成功した場合、読み込んだバイト数を返す (0 はファイル
		 * の終りを意味する)。エラーの場合は、-1 が返され、
		 * errno が適切に設定される。
		 */
		len = read(fd, buf, sizeof(buf));
		if (len <= 0) {
			/* ファイルの終わりに達した? */
			if (len == 0)
				break;

			/* エラーが発生した */
			perror("read");
			ret = EXIT_FAILURE;
			break;
		}

		/*
		 * 成功した場合、書き込まれたバイト数が返される (ゼロは
		 * 何も書き込まれなかったことを示す)。エラーならば -1
		 * が返され、errno が適切に設定される。
		 */
		len = write(1, buf, len);
		/* エラーが発生した? */
		if (len == -1) {
			perror("write");
			ret = EXIT_FAILURE;
			break;
		}
	}

	/*
	 * close() は成功した場合は 0 を返す。エラーが発生した場合は
	 * -1 を返して、 errno を適切に設定する。
	 */
	if (close(fd) == -1) {
		perror("close");
		ret = EXIT_FAILURE;
	}

	return ret;
}
