import csv
import sys
import os
import copy

# 表示する文字数
DISP_WIDTH = 60  # 幅
DISP_HEIGHT = 17 # 高さ

# CSVデータ1行の要素数
COLUMN_NUM = 11

line = 0   # 一画面に表示した行数
count = 0  # 表示したデータ総数


def printline(csvline):

    """ 行表示関数

    Args:
        csvline(stringの配列): 表示する1行分のCSVデータ

    """

    global line
    global count

    # CSVデータの各要素を表示
    if csvline is not None:
        # データがない場合表示しない
        if len(csvline) < 1:
            return

        #各要素を表示フォーマットに展開
        buf = "%6d %s %s %s %s %s %s %s %s %s %s %s" % \
                (count, csvline[0], csvline[1], csvline[2], csvline[3], csvline[4], \
                 csvline[5], csvline[6], csvline[7], csvline[8], csvline[9], csvline[10])
        count += 1
    else:
        # CSVデータの総数を表示
        # データ総数を表示フォーマットに展開
        buf = "Count: %6d" % (count)

    # 今回追加される表示行数を計算
    newline = int((len(buf) + DISP_WIDTH - 1) / DISP_WIDTH)
    # 1画面を超える場合、入力があるまで一時停止
    if line + newline >= DISP_HEIGHT:
        input()
        # 表示行数を初期化
        line = 0
    # 実際に表示する
    print(buf)
    # 表示行数を更新
    line += newline

if __name__ == "__main__":
    # 引数が指定されなかった場合、usage表示して終了
    if len(sys.argv) < 2:
        print("Usage: %s <csvfile>" % sys.argv[0])
        sys.exit(os.EX_OK)

    # CSVファイルオープン
    with open(sys.argv[1]) as csvfile:
        pzip = "00000000"
        csvline = [] # CSVデータ初期化
        # CSVファイルから1行読み込む
        csvreader = csv.reader(csvfile)
        for row in csvreader:
            # 要素数が不足している場合、次行にスキップ
            if len(row) < COLUMN_NUM:
                continue

            # 新しいデータの場合
            if pzip != row[1]:
                printline(csvline)
                csvline = copy.copy(row)
            else:
                # 既存データへの追加の場合
                # 町域名の続きを追加
                csvline[2] += row[2]
            pzip = row[1]

        # 保持済みのデータを表示
        printline(csvline)
        # データ総数を表示
        printline(None)
