import csv
import configparser
import sys
import os
import copy
import json

# 表示する文字数
DISP_WIDTH = 60  # 幅
DISP_HEIGHT = 17 # 高さ

# CSVデータ1行の要素数
COLUMN_NUM = 11

line = 0   # 一画面に表示した行数
count = 0  # 表示したデータ総数

conf = configparser.ConfigParser() # 設定ファイル管理オブジェクト

def printline(csvline):

    """ 行表示関数

    Args:
        csvline(stringの配列): 表示する1行分のCSVデータ

    """

    global line
    global count

    # CSVデータの各要素を表示
    if csvline is not None:
        # データがない場合表示しない
        if len(csvline) < 1:
            return

        # 全国地方公共団体コード条件設定があり、一致しなかったら表示しない
        if int(conf['data']['Code']) >= 0 and csvline[0] != conf['data']['Code']:
            return
        # 郵便番号条件設定があり、部分一致しなかったら表示しない
        if conf['data']['Zipcode'] != "" and conf['data']['Zipcode'] not in csvline[1]:
            return
        # 町域名条件設定があり、部分一致しなかったら表示しない
        if conf['data']['Street'] != "" and conf['data']['Street'].lower() not in csvline[2].lower():
            return
        # 市区町村名条件設定があり、部分一致しなかったら表示しない
        if conf['data']['City'] != "" and conf['data']['City'].lower() not in csvline[3].lower():
            return
        # 都道府県条件設定があり、部分一致しなかったら表示しない */
        if conf['data']['Pref'] != "" and conf['data']['Pref'].lower() not in csvline[4].lower():
            return

        for idx, flag in enumerate(json.loads(conf['data']['Flag'])):
            if int(flag) >= 0 and flag != csvline[5 + idx]:
                return

        #各要素を表示フォーマットに展開
        buf = "%6d %s %s %s %s %s %s %s %s %s %s %s" % \
                (count, csvline[0], csvline[1], csvline[2], csvline[3], csvline[4], \
                 csvline[5], csvline[6], csvline[7], csvline[8], csvline[9], csvline[10])
        count += 1
    else:
        # CSVデータの総数を表示
        # 総数表示無効なら表示しない
        if not conf['control']['Count']:
            return
        # データ総数を表示フォーマットに展開
        buf = "Count: %6d" % (count)

    # 今回追加される表示行数を計算
    disp_width = int(conf['display']['Width'])
    disp_height = int(conf['display']['Height'])
    newline = int((len(buf) + disp_width - 1) / disp_width)
    # 1画面を超える場合、入力があるまで一時停止
    if line + newline >= disp_height:
        # 一時停止有効なら
        if conf['control']['Pause']:
            input()
        # 表示行数を初期化
        line = 0
    # 実際に表示する
    print(buf)
    # 表示行数を更新
    line += newline


def readconf(conf_file_name):

    """ confファイル読み込み関数

    Args:
        conf_file_name(string): confファイルのファイル名

    """

    global conf
    if not os.path.exists("./" + conf_file_name):
        # ファイルが存在していない場合は新規作成
        # デフォルト設定
        conf['display'] = {
            'Width': 60,
            'Height': 17
        }
        conf['control'] = {
            'Pause': True,
            'Count': True
        }
        conf['data'] = {
            'Code': -1,
            'Zipcode': "",
            'Street': "",
            'City': "",
            'Pref': "",
            'Flag': [-1, -1, -1, -1, -1, -1]
        }
        with open(conf_file_name, 'w') as conf_file:
            conf.write(conf_file)
    else :
        conf.read(conf_file_name)

if __name__ == "__main__":
    # 引数が指定されなかった場合、usage表示して終了
    if len(sys.argv) < 2:
        print("Usage: %s <csvfile>" % sys.argv[0])
        sys.exit(os.EX_OK)

    # confファイルを読み込み
    readconf(os.path.splitext(sys.argv[0])[0] + ".conf")

    # CSVファイルオープン
    with open(sys.argv[1]) as csvfile:
        csvreader = csv.reader(csvfile)
        pzip = "00000000"
        csvline = [] # CSVデータ初期化
        # CSVファイルから1行読み込む
        for row in csvreader:
            # 要素数が不足している場合、次行にスキップ
            if len(row) < COLUMN_NUM:
                continue

            # 新しいデータの場合
            if pzip != row[1]:
                printline(csvline)
                csvline = copy.copy(row)
            else:
                # 既存データへの追加の場合
                # 町域名の続きを追加
                csvline[2] += row[2]
            pzip = row[1]

        # 保持済みのデータを表示
        printline(csvline)
        # データ総数を表示
        printline(None)
