import csv
import sys
import os
import copy
from xml.etree import ElementTree # XML操作用パッケージのインポート
import xml.dom.minidom            # XML操作用パッケージのインポート

# 表示する文字数
DISP_WIDTH = 60  # 幅
DISP_HEIGHT = 17 # 高さ

# CSVデータ1行の要素数
COLUMN_NUM = 11

line = 0   # 一画面に表示した行数
count = 0  # 表示したデータ総数

conf = {} # 設定ファイル管理オブジェクト

def printline(csvline):

    """ 行表示関数

    Args:
        csvline(stringの配列): 表示する1行分のCSVデータ

    """

    global line
    global count

    # CSVデータの各要素を表示
    if csvline is not None:
        # データがない場合表示しない
        if len(csvline) < 1:
            return

        # 全国地方公共団体コード条件設定があり、一致しなかったら表示しない
        if int(conf['data']['Code']) >= 0 and csvline[0] != conf['data']['Code']:
            return
        # 郵便番号条件設定があり、部分一致しなかったら表示しない
        if conf['data']['Zipcode'] != None and conf['data']['Zipcode'] not in csvline[1]:
            return
        # 町域名条件設定があり、部分一致しなかったら表示しない
        if conf['data']['Street'] != None and conf['data']['Street'].lower() not in csvline[2].lower():
            return
        # 市区町村名条件設定があり、部分一致しなかったら表示しない
        if conf['data']['City'] != None and conf['data']['City'].lower() not in csvline[3].lower():
            return
        # 都道府県条件設定があり、部分一致しなかったら表示しない */
        if conf['data']['Pref'] != None and conf['data']['Pref'].lower() not in csvline[4].lower():
            return

        for idx, flag in enumerate(conf['data']['Flag']):
            if flag >= 0 and flag != csvline[5 + idx]:
                return

        #各要素を表示フォーマットに展開
        buf = "%6d %s %s %s %s %s %s %s %s %s %s %s" % \
                (count, csvline[0], csvline[1], csvline[2], csvline[3], csvline[4], \
                 csvline[5], csvline[6], csvline[7], csvline[8], csvline[9], csvline[10])
        count += 1
    else:
        # CSVデータの総数を表示
        # 総数表示無効なら表示しない
        if not conf['control']['Count']:
            return
        # データ総数を表示フォーマットに展開
        buf = "Count: %6d" % (count)

    # 今回追加される表示行数を計算
    disp_width = conf['display']['Width']
    disp_height = conf['display']['Height']
    newline = int((len(buf) + disp_width - 1) / disp_width)
    # 1画面を超える場合、入力があるまで一時停止
    if line + newline >= disp_height:
        # 一時停止有効なら
        if conf['control']['Pause']:
            input()
        # 表示行数を初期化
        line = 0
    # 実際に表示する
    print(buf)
    # 表示行数を更新
    line += newline


def readconf(conf_file_name):

    """ confファイル読み込み関数

    Args:
        conf_file_name(string): confファイルのファイル名

    """

    global conf
    if not os.path.exists("./" + conf_file_name):
        # ファイルが存在していない場合は新規作成
        # デフォルト設定
        conf['display'] = {
            'Width': 60,
            'Height': 17
        }
        conf['control'] = {
            'Pause': True,
            'Count': True
        }
        conf['data'] = {
            'Code': -1,
            'Zipcode': "",
            'Street': "",
            'City': "",
            'Pref': "",
            'Flag': [-1, -1, -1, -1, -1, -1]
        }

        # conf要素(ルート)の作成
        conf_elem = ElementTree.Element("conf")

        # display要素の作成とその下にある子要素の作成
        display_elem = ElementTree.SubElement(conf_elem, "display")
        width_elem = ElementTree.SubElement(display_elem, "width")
        width_elem.text = str(conf['display']['Width'])
        height_elem = ElementTree.SubElement(display_elem, "height")
        height_elem.text = str(conf['display']['Height'])

        # control要素の作成とその下にある子要素の作成
        control_elem = ElementTree.SubElement(conf_elem, "control")
        pause_elem = ElementTree.SubElement(control_elem, "pause")
        pause_elem.text = str(conf['control']['Pause'])
        count_elem = ElementTree.SubElement(control_elem, "count")
        count_elem.text = str(conf['control']['Count'])

        # data要素の作成とその下にある子要素の作成
        data_elem = ElementTree.SubElement(conf_elem, "data")
        code_elem = ElementTree.SubElement(data_elem, "code")
        code_elem.text = str(conf['data']['Code'])
        zipcode_elem = ElementTree.SubElement(data_elem, "zipcode")
        zipcode_elem.text = ""
        street_elem = ElementTree.SubElement(data_elem, "street")
        street_elem.text = ""
        city_elem = ElementTree.SubElement(data_elem, "city")
        city_elem.text = ""
        pref_elem = ElementTree.SubElement(data_elem, "pref")
        pref_elem.text = ""
        flag_elem = ElementTree.SubElement(data_elem, "flag")
        flag_elem.text = "-1,-1,-1,-1,-1,-1"

        # ファイルに書き出したときに見やすくするため整形する
        doc = xml.dom.minidom.parseString(ElementTree.tostring(conf_elem, "utf-8"))
        with open(conf_file_name, "w") as conf_file:
            doc.writexml(conf_file, newl="\n", indent="", addindent="    ")
    else :
        # XML形式のデータとして読み込む
        conf_elem = ElementTree.parse(conf_file_name)
        conf['display'] = {}
        conf['display']['Width'] = int(conf_elem.find(".//width").text)
        conf['display']['Height'] = int(conf_elem.find(".//height").text)
        conf['control'] = {}
        conf['control']['Pause'] = bool(conf_elem.find(".//pause").text)
        conf['control']['Count'] = bool(conf_elem.find(".//count").text)
        conf['data'] = {}
        conf['data']['Code'] = int(conf_elem.find(".//code").text)
        conf['data']['Zipcode'] = conf_elem.find(".//zipcode").text
        conf['data']['Street'] = conf_elem.find(".//street").text
        conf['data']['City'] = conf_elem.find(".//city").text
        conf['data']['Pref'] = conf_elem.find(".//pref").text

        flag = []
        for val in conf_elem.find(".//flag").text.split(","):
            flag.append(int(val))
        conf['data']['Flag'] = flag

if __name__ == "__main__":
    # 引数が指定されなかった場合、usage表示して終了
    if len(sys.argv) < 2:
        print("Usage: %s <csvfile>" % sys.argv[0])
        sys.exit(os.EX_OK)

    # confファイルを読み込み
    readconf(os.path.splitext(sys.argv[0])[0] + ".xml")

    # CSVファイルオープン
    with open(sys.argv[1]) as csvfile:
        csvreader = csv.reader(csvfile)
        pzip = "00000000"
        csvline = [] # CSVデータ初期化
        # CSVファイルから1行読み込む
        for row in csvreader:
            # 要素数が不足している場合、次行にスキップ
            if len(row) < COLUMN_NUM:
                continue

            # 新しいデータの場合
            if pzip != row[1]:
                printline(csvline)
                csvline = copy.copy(row)
            else:
                # 既存データへの追加の場合
                # 町域名の続きを追加
                csvline[2] += row[2]
            pzip = row[1]

        # 保持済みのデータを表示
        printline(csvline)
        # データ総数を表示
        printline(None)
